'use strict'

import IdleJs from 'idle-js'
import IdleView from "./src/Idle"

export default {
  IdleView,

  install (Vue, options) {
    const {
      eventEmitter,
      store,
      moduleName = 'idleVue',

      idleTime = 60 * 1000,
      events = ['mousemove', 'keydown', 'mousedown', 'touchstart'],
      keepTracking = true,
      startAtIdle = true
    } = options || {}

    if (!eventEmitter && !store) {
      throw Error("Either `eventEmitter` or `store` must be passed in options")
    }

    const onIdleStr = `${moduleName}_onIdle`
    const onActiveStr = `${moduleName}_onActive`

    store && store.registerModule(moduleName, {
      state: { isIdle: startAtIdle },

      mutations: {
        [`${moduleName}/IDLE_CHANGED`]: function (state, isIdle) {
          state.isIdle = isIdle
        }
      }
    })

    const idle = new IdleJs({
      idle: idleTime,
      events,
      keepTracking,
      startAtIdle,

      onIdle: () => {
        eventEmitter && eventEmitter.$emit(onIdleStr)
        store && store.commit(`${moduleName}/IDLE_CHANGED`, true)
      },
      onActive: () => {
        eventEmitter && eventEmitter.$emit(onActiveStr)
        store && store.commit(`${moduleName}/IDLE_CHANGED`, false)
      }
    })
    idle.start()

    Vue.component("IdleView", IdleView)

    Vue.mixin({
      data () { return {
        [onIdleStr]: null,
        [onActiveStr]: null
      }},
      created () {
        const options = this.$options

        this[onIdleStr] = () => options.onIdle && options.onIdle()
        this[onActiveStr] = () => options.onActive && options.onActive()
        eventEmitter && eventEmitter.$on(onIdleStr, this[onIdleStr])
        eventEmitter && eventEmitter.$on(onActiveStr, this[onActiveStr])
      },
      destroyed () {
        eventEmitter && eventEmitter.$off(onIdleStr, this[onIdleStr])
        eventEmitter && eventEmitter.$off(onActiveStr, this[onActiveStr])
      },
      computed: {
        isAppIdle () {
          return store && store.state.idle
        }
      }
    })
  }
}
